@@ -51,11 +51,12 @@ getopt( 'wsqvmt', \%opt );
 my $xs = Novel::Robot->new();
 $xs->set_packer($opt{t} || 'TXT');
 $xs->set_parser($opt{w} || $opt{s});
+
 my $books_ref;
 if($opt{w}){
     #writer
     my $writer_ref = $xs->get_writer_ref($opt{w});
-    $books_ref = $writer_ref->{series};
+    $books_ref = $writer_ref->{booklist};
 }elsif($opt{q}){
     #query
     my $keyword = decode( locale => $opt{q});
@@ -65,7 +66,7 @@ if($opt{w}){
 
 my $select = $opt{m} ? $xs->select_book($books_ref) : $books_ref;
 for my $r (@$select) {
-    my $u = $r->[2];
+    my $u = $r->{url};
     print "\rselect : $u\n";
     $xs->get_book($u);
 } ## end for my $r (@$select)
