@@ -60,8 +60,8 @@ my %packer_opt = (
     'usr'      => $opt{u},
     'passwd'   => $opt{p},
 );
-$packer_opt{tag}      = decode( locale => $opt{t} ) if ( $opt{t} );
-$packer_opt{category} = decode( locale => $opt{c} ) if ( $opt{c} );
+$packer_opt{tag}      = [ split ',', decode( locale => $opt{t} ) ] if ( $opt{t} );
+$packer_opt{category} = [ split ',' , decode( locale => $opt{c} ) ] if ( $opt{c} );
 
 $xs->set_packer( 'WordPress', \%packer_opt );
 $xs->get_book($index_url);
